﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.EvolutionTracking;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [StorableClass]
  [Item("SymbolicDataAnalysisExpressionAfterCrossoverOperator", "An operator that compares the crossover child with the parent and saves the swapped fragment.")]
  public class SymbolicDataAnalysisExpressionAfterCrossoverOperator : AfterCrossoverOperator<ISymbolicExpressionTree> {
    public SymbolicDataAnalysisExpressionAfterCrossoverOperator() { }

    protected SymbolicDataAnalysisExpressionAfterCrossoverOperator(SymbolicDataAnalysisExpressionAfterCrossoverOperator original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SymbolicDataAnalysisExpressionAfterCrossoverOperator(this, cloner);
    }

    [StorableConstructor]
    protected SymbolicDataAnalysisExpressionAfterCrossoverOperator(bool deserializing) : base(deserializing) { }

    public override IOperation Apply() {
      var childVertex = GenealogyGraph.GetByContent(ChildParameter.ActualValue);
      var arcs = childVertex.InArcs.ToList();
      var nodes0 = (List<ISymbolicExpressionTreeNode>)arcs[0].Data;
      var nodes1 = (List<ISymbolicExpressionTreeNode>)arcs[1].Data;
      var childNodes = childVertex.Data.IterateNodesPrefix().ToList();
      IFragment<ISymbolicExpressionTreeNode> fragment = null;
      for (int i = 0; i < Math.Min(nodes0.Count, childNodes.Count); ++i) {
        if (nodes0[i] == childNodes[i]) continue;
        fragment = new Fragment<ISymbolicExpressionTreeNode> {
          Root = childNodes[i],
          Index1 = i,
        };
        fragment.Index2 = nodes1.IndexOf(fragment.Root);
        break;
      }

      // if the fragment is null it means that nodes0 is identical with childNodes (in other words crossover did nothing)
      arcs[0].Data = null;
      arcs[1].Data = fragment;

      return base.Apply();
    }
  }
}
